package ru.landgrafhomyak.kotlin.kmp_gradle_build_helper.plugin

import kotlin.contracts.InvocationKind
import kotlin.contracts.contract
import kotlin.reflect.KClass
import org.gradle.api.Project
import org.gradle.api.publish.PublishingExtension
import org.jetbrains.kotlin.gradle.dsl.KotlinMultiplatformExtension
import ru.landgrafhomyak.kotlin.kmp_gradle_build_helper.plugin.SetupBuildScript.PluginIDs

class XomrkDsl internal constructor(private val project: Project) {
	@Suppress("FunctionName")
	private fun <T : Any> _wrapExtension(
		@Suppress("SameParameterValue") pluginId: String,
		extensionClass: KClass<T>,
		config: T.() -> Unit
	) {
		contract {
			callsInPlace(config, InvocationKind.EXACTLY_ONCE)
		}

		if (null == this.project.plugins.findPlugin(pluginId)) {
			this.project.plugins.apply(pluginId)
		}

		val dsl = this.project.extensions.findByType(extensionClass.java) ?: throw RuntimeException("Can't get extension")
		config(dsl)

	}

	fun kotlin(config: KotlinMultiplatformExtension.() -> Unit) {
		contract {
			callsInPlace(config, InvocationKind.EXACTLY_ONCE)
		}

		this._wrapExtension(PluginIDs.KOTLIN_MULTIPLATFORM, KotlinMultiplatformExtension::class, config)
	}

	fun publishing(config: PublishingExtension.() -> Unit) {
		this._wrapExtension(PluginIDs.MAVEN_PUBLISHING, PublishingExtension::class, config)

	}
}